=== modified file 'lib/canonical/launchpad/interfaces/_schema_circular_imports.py'
--- lib/canonical/launchpad/interfaces/_schema_circular_imports.py	2011-04-27 15:12:08 +0000
+++ lib/canonical/launchpad/interfaces/_schema_circular_imports.py	2011-05-09 15:04:00 +0000
@@ -490,6 +490,8 @@
     IDistroSeries, 'deriveDistroSeries', 'distribution', IDistribution)
 patch_collection_return_type(
     IDistroSeries, 'getDerivedSeries', IDistroSeries)
+patch_collection_return_type(
+    IDistroSeries, 'getParentSeries', IDistroSeries)
 patch_plain_parameter_type(
     IDistroSeries, 'getDifferencesTo', 'parent_series', IDistroSeries)
 patch_choice_parameter_type(
@@ -856,8 +858,8 @@
 patch_entry_explicit_version(IDistroSeries, 'beta')
 patch_operations_explicit_version(
     IDistroSeries, 'beta', "deriveDistroSeries", "getDerivedSeries",
-    "getDistroArchSeries", "getPackageUploads", "getSourcePackage",
-    "newMilestone")
+    "getParentSeries", "getDistroArchSeries", "getPackageUploads",
+    "getSourcePackage", "newMilestone")
 
 # IDistroSeriesDifference
 patch_entry_explicit_version(IDistroSeriesDifference, 'beta')

=== modified file 'lib/lp/registry/browser/distroseries.py'
--- lib/lp/registry/browser/distroseries.py	2011-05-06 10:45:50 +0000
+++ lib/lp/registry/browser/distroseries.py	2011-05-09 15:04:00 +0000
@@ -350,7 +350,35 @@
             self.context.datereleased = UTC_NOW
 
 
-class DistroSeriesView(LaunchpadView, MilestoneOverlayMixin):
+class DerivedDistroSeriesMixin():
+
+    @cachedproperty
+    def has_unique_parent(self):
+        return len(self.context.getParentSeries()) == 1
+
+    @cachedproperty
+    def unique_parent(self):
+        if self.has_unique_parent:
+            return self.context.getParentSeries()[0]
+        else:
+            None
+
+    @cachedproperty
+    def number_of_parents(self):
+        return len(self.context.getParentSeries())
+
+    @cachedproperty
+    def parent_name(self):
+        if self.has_unique_parent:
+            parent_name = ("parent series '%s'" %
+                self.unique_parent.displayname)
+        else:
+            parent_name = 'a parent series'
+        return parent_name
+
+
+class DistroSeriesView(LaunchpadView, MilestoneOverlayMixin,
+                       DerivedDistroSeriesMixin):
 
     def initialize(self):
         super(DistroSeriesView, self).initialize()
@@ -435,15 +463,6 @@
         return self._num_differences(
             DistroSeriesDifferenceType.UNIQUE_TO_DERIVED_SERIES)
 
-    @cachedproperty
-    def parent_series(self):
-        # XXX StevenK: This will currently throw NotOneError up the
-        # callstack if the child has more than one parent. This is okay
-        # *for now*, so we can get away with no UI changes.
-        dsp = getUtility(IDistroSeriesParentSet).getByDerivedSeries(
-            self.context).one()
-        return dsp.parent_series
-
 
 class DistroSeriesEditView(LaunchpadEditFormView, SeriesStatusMixin):
     """View class that lets you edit a DistroSeries object.
@@ -673,7 +692,7 @@
         higher_term = SimpleTerm(
             HIGHER_VERSION_THAN_PARENT,
             HIGHER_VERSION_THAN_PARENT,
-            "Blacklisted packages with a higher version than in '%s'"
+            "Blacklisted packages with a higher version than in %s"
                 % parent_name)
         voc.insert(2, higher_term)
     return SimpleVocabulary(tuple(voc))
@@ -706,7 +725,8 @@
 
 
 class DistroSeriesDifferenceBaseView(LaunchpadFormView,
-                                     PackageCopyingMixin):
+                                     PackageCopyingMixin,
+                                     DerivedDistroSeriesMixin):
     """Base class for all pages presenting differences between
     a derived series and its parent."""
     schema = IDifferencesFormSchema
@@ -741,10 +761,14 @@
         return NotImplementedError()
 
     def setupPackageFilterRadio(self):
+        if self.has_unique_parent:
+            parent_name = "'%s'" % self.unique_parent.displayname
+        else:
+            parent_name = 'parent'
         return form.Fields(Choice(
             __name__='package_type',
             vocabulary=make_package_type_vocabulary(
-                self.parent_series.displayname,
+                parent_name,
                 self.search_higher_parent_option),
             default=DEFAULT_PACKAGE_TYPE,
             required=True))
@@ -761,9 +785,8 @@
             self.form_fields)
         check_permission('launchpad.Edit', self.context)
         terms = [
-            SimpleTerm(diff, diff.source_package_name.name,
-                diff.source_package_name.name)
-                for diff in self.cached_differences.batch]
+            SimpleTerm(diff, diff.id)
+                    for diff in self.cached_differences.batch]
         diffs_vocabulary = SimpleVocabulary(terms)
         choice = self.form_fields['selected_differences'].field.value_type
         choice.vocabulary = diffs_vocabulary
@@ -895,15 +918,6 @@
                         DistroSeriesDifferenceStatus.BLACKLISTED_CURRENT))
             return not differences.is_empty()
 
-    @cachedproperty
-    def parent_series(self):
-        # XXX StevenK: This will currently throw NotOneError up the
-        # callstack if the child has more than one parent. This is okay
-        # *for now*, so we can get away with no UI changes.
-        dsp = getUtility(IDistroSeriesParentSet).getByDerivedSeries(
-            self.context).one()
-        return dsp.parent_series
-
 
 class DistroSeriesLocalDifferencesView(DistroSeriesDifferenceBaseView,
                                        LaunchpadFormView):
@@ -917,9 +931,13 @@
 
     def initialize(self):
         # Update the label for sync action.
+        if self.has_unique_parent:
+            parent_name = "'%s'" % self.unique_parent.displayname
+        else:
+            parent_name = 'Parent'
         self.initialize_sync_label(
             "Sync Selected %s Versions into %s" % (
-                self.parent_series.displayname,
+                parent_name,
                 self.context.displayname,
                 ))
         super(DistroSeriesLocalDifferencesView, self).initialize()
@@ -928,23 +946,26 @@
     def explanation(self):
         return structured(
             "Source packages shown here are present in both %s "
-            "and the parent series, %s, but are different somehow. "
+            "and %s, but are different somehow. "
             "Changes could be in either or both series so check the "
-            "versions (and the diff if necessary) before syncing the %s "
+            "versions (and the diff if necessary) before syncing the parent "
             'version (<a href="/+help/soyuz/derived-series-syncing.html" '
-            'target="help">Read more about syncing from the parent series'
+            'target="help">Read more about syncing from a parent series'
             '</a>).',
             self.context.displayname,
-            self.parent_series.fullseriesname,
-            self.parent_series.displayname)
+            self.parent_name)
 
     @property
     def label(self):
+        if self.has_unique_parent:
+            parent_name = self.parent_name
+        else:
+            parent_name = 'parent series'
         return (
-            "Source package differences between '%s' and "
-            "parent series '%s'" % (
+            "Source package differences between '%s' and"
+            " %s" % (
                 self.context.displayname,
-                self.parent_series.displayname,
+                parent_name,
                 ))
 
     @action(_("Update"), name="update")
@@ -972,7 +993,7 @@
     def initialize(self):
         # Update the label for sync action.
         self.initialize_sync_label(
-            "Include Selected packages into into %s" % (
+            "Include Selected packages into %s" % (
                 self.context.displayname,
                 ))
         super(DistroSeriesMissingPackagesView, self).initialize()
@@ -981,17 +1002,17 @@
     def explanation(self):
         return structured(
             "Packages that are listed here are those that have been added to "
-            "the specific packages %s that were used to create %s. They are "
-            "listed here so you can consider including them in %s.",
-            self.parent_series.displayname,
+            "the specific set of packages in %s that were used to create %s. "
+            "They are listed here so you can consider including them in %s.",
+            self.parent_name,
             self.context.displayname,
             self.context.displayname)
 
     @property
     def label(self):
         return (
-            "Packages in parent series '%s' but not in '%s'" % (
-                self.parent_series.displayname,
+            "Packages in %s but not in '%s'" % (
+                self.parent_name,
                 self.context.displayname,
                 ))
 
@@ -1024,16 +1045,16 @@
     def explanation(self):
         return structured(
             "Packages that are listed here are those that have been added to "
-            "%s but are not yet part of the parent series %s.",
+            "%s but are not yet part of %s.",
             self.context.displayname,
-            self.parent_series.displayname)
+            self.parent_name)
 
     @property
     def label(self):
         return (
-            "Packages in '%s' but not in parent series '%s'" % (
+            "Packages in '%s' but not in %s" % (
                 self.context.displayname,
-                self.parent_series.displayname,
+                self.parent_name,
                 ))
 
     @action(_("Update"), name="update")

=== modified file 'lib/lp/registry/browser/tests/test_distroseries.py'
--- lib/lp/registry/browser/tests/test_distroseries.py	2011-05-08 13:41:34 +0000
+++ lib/lp/registry/browser/tests/test_distroseries.py	2011-05-09 15:04:00 +0000
@@ -128,29 +128,42 @@
 
     layer = DatabaseFunctionalLayer
 
-    def _setupDifferences(self, name, parent_name, nb_diff_versions,
+    def _setupDifferences(self, name, parent_names, nb_diff_versions,
                           nb_diff_child, nb_diff_parent):
-        # Helper to create DSD of the different types.
+        # Helper to create DSDs of the different types.
         derived_series = self.factory.makeDistroSeries(name=name)
-        parent_series = self.factory.makeDistroSeries(name=parent_name)
-        self.factory.makeDistroSeriesParent(
-            derived_series=derived_series, parent_series=parent_series)
         self.simple_user = self.factory.makePerson()
+        # parent_names can be a list of parent names or a single name
+        # for a single parent (e.g. ['parent1_name', 'parent2_name'] or
+        # 'parent_name').
+        # If multiple parents are created, the DSDs will be created with
+        # the first one.
+        if type(parent_names) == str:
+            parent_names = [parent_names]
+        dsps = []
+        for parent_name in parent_names:
+            parent_series = self.factory.makeDistroSeries(name=parent_name)
+            dsps.append(self.factory.makeDistroSeriesParent(
+                derived_series=derived_series, parent_series=parent_series))
+        first_parent_series = dsps[0].parent_series
         for i in range(nb_diff_versions):
             diff_type = DistroSeriesDifferenceType.DIFFERENT_VERSIONS
             self.factory.makeDistroSeriesDifference(
                 derived_series=derived_series,
-                difference_type=diff_type)
+                difference_type=diff_type,
+                parent_series=first_parent_series)
         for i in range(nb_diff_child):
             diff_type = DistroSeriesDifferenceType.MISSING_FROM_DERIVED_SERIES
             self.factory.makeDistroSeriesDifference(
                 derived_series=derived_series,
-                difference_type=diff_type)
+                difference_type=diff_type,
+                parent_series=first_parent_series)
         for i in range(nb_diff_parent):
             diff_type = DistroSeriesDifferenceType.UNIQUE_TO_DERIVED_SERIES
             self.factory.makeDistroSeriesDifference(
                 derived_series=derived_series,
-                difference_type=diff_type)
+                difference_type=diff_type,
+                parent_series=first_parent_series)
         return derived_series
 
     def test_differences_no_flag_no_portlet(self):
@@ -207,6 +220,33 @@
 
         self.assertThat(html_content, portlet_display)
 
+    def test_differences_portlet_all_differences_multiple_parents(self):
+        # The difference portlet shows the differences with the multiple
+        # parent series.
+        set_derived_series_ui_feature_flag(self)
+        derived_series = self._setupDifferences(
+            'deri', ['sid1', 'sid2'], 0, 1, 0)
+        portlet_display = soupmatchers.HTMLContains(
+            soupmatchers.Tag(
+                'Derivation portlet header', 'h2',
+                text='Derived from 2 parents'),
+            soupmatchers.Tag(
+                'Parent diffs link', 'a',
+                text=re.compile('\s*1 package only in a parent series\s*'),
+                attrs={'href': re.compile('.*/\+missingpackages')}))
+
+        with person_logged_in(self.simple_user):
+            view = create_initialized_view(
+                derived_series,
+                '+index',
+                principal=self.simple_user)
+            # XXX rvb 2011-04-12 bug=758649: LaunchpadTestRequest overrides
+            # self.features to NullFeatureController.
+            view.request.features = get_relevant_feature_controller()
+            html_text = view()
+
+        self.assertThat(html_text, portlet_display)
+
     def test_differences_portlet_no_differences(self):
         # The difference portlet displays 'No differences' if there is no
         # differences with the parent.
@@ -405,13 +445,19 @@
 
         self.assertThat(html_content, parent_packagesets)
 
-    def _create_child_and_parent(self):
+    def _createChildAndParent(self):
         derived_series = self.factory.makeDistroSeries(name='derilucid')
         parent_series = self.factory.makeDistroSeries(name='lucid')
         self.factory.makeDistroSeriesParent(
             derived_series=derived_series, parent_series=parent_series)
         return (derived_series, parent_series)
 
+    def _createChildAndParents(self, other_parent_series=None):
+        derived_series, parent_series = self._createChildAndParent()
+        self.factory.makeDistroSeriesParent(
+            derived_series=derived_series, parent_series=other_parent_series)
+        return (derived_series, parent_series)
+
 
 class TestDistroSeriesLocalDifferences(
     DistroSeriesDifferenceMixin, TestCaseWithFactory):
@@ -429,7 +475,7 @@
         # Test that the page includes the filter form if differences
         # are present
         login_person(self.simple_user)
-        derived_series, parent_series = self._create_child_and_parent()
+        derived_series, parent_series = self._createChildAndParent()
         self.factory.makeDistroSeriesDifference(
             derived_series=derived_series)
 
@@ -445,7 +491,7 @@
         # Test that the page doesn't includes the filter form if no
         # differences are present
         login_person(self.simple_user)
-        derived_series, parent_series = self._create_child_and_parent()
+        derived_series, parent_series = self._createChildAndParent()
 
         view = create_initialized_view(
             derived_series, '+localpackagediffs', principal=self.simple_user)
@@ -499,13 +545,24 @@
             html_content, packageset_text, 'parent-packagesets',
             'Parent packagesets')
 
-    def test_queries(self):
+
+class TestDistroSeriesLocalDifferencesPerformance(DistroSeriesDifferenceMixin,
+                                                  TestCaseWithFactory):
+    """Test the distroseries +localpackagediffs page's performance."""
+
+    layer = DatabaseFunctionalLayer
+
+    def setUp(self):
+        super(TestDistroSeriesLocalDifferencesPerformance,
+             self).setUp('foo.bar@canonical.com')
+        set_derived_series_ui_feature_flag(self)
+        self.simple_user = self.factory.makePerson()
+
+    def _assertQueryCount(self, derived_series):
         # With no DistroSeriesDifferences the query count should be low and
         # fairly static. However, with some DistroSeriesDifferences the query
         # count will be higher, but it should remain the same no matter how
         # many differences there are.
-        dsp = self.factory.makeDistroSeriesParent()
-        derived_series = dsp.derived_series
         ArchivePermission(
             archive=derived_series.main_archive, person=self.simple_user,
             component=getUtility(IComponentSet)["main"],
@@ -613,13 +670,26 @@
             recorder3, HasQueryCount(
                 LessThan(compromise_statement_count + 1)))
 
-
-class TestDistroSeriesLocalDifferencesZopeless(TestCaseWithFactory):
+    def test_queries_single_parent(self):
+        dsp = self.factory.makeDistroSeriesParent()
+        derived_series = dsp.derived_series
+        self._assertQueryCount(derived_series)
+
+    def test_queries_multiple_parents(self):
+        dsp = self.factory.makeDistroSeriesParent()
+        derived_series = dsp.derived_series
+        self.factory.makeDistroSeriesParent(
+            derived_series=derived_series)
+        self._assertQueryCount(derived_series)
+
+
+class TestDistroSeriesLocalDifferencesZopeless(DistroSeriesDifferenceMixin,
+                                               TestCaseWithFactory):
     """Test the distroseries +localpackagediffs view."""
 
     layer = LaunchpadZopelessLayer
 
-    def _create_child_and_parent(self):
+    def _createChildAndParent(self):
         parent_series = self.factory.makeDistroSeries(name='lucid')
         derived_series = self.factory.makeDistroSeries(name='derilucid')
         self.factory.makeDistroSeriesParent(
@@ -629,7 +699,7 @@
     def test_view_redirects_without_feature_flag(self):
         # If the feature flag soyuz.derived-series-ui.enabled is not set the
         # view simply redirects to the derived series.
-        derived_series, parent_series = self._create_child_and_parent()
+        derived_series, parent_series = self._createChildAndParent()
 
         self.assertIs(
             None, getFeatureFlag('soyuz.derived-series-ui.enabled'))
@@ -643,7 +713,7 @@
 
     def test_label(self):
         # The view label includes the names of both series.
-        derived_series, parent_series = self._create_child_and_parent()
+        derived_series, parent_series = self._createChildAndParent()
 
         view = create_initialized_view(
             derived_series, '+localpackagediffs')
@@ -653,10 +723,23 @@
             "parent series 'Lucid'",
             view.label)
 
+    def test_label_multiple_parents(self):
+        # If the series has multiple parents, the view label mentions
+        # the generic term 'parent series'.
+        derived_series, parent_series = self._createChildAndParents()
+
+        view = create_initialized_view(
+            derived_series, '+localpackagediffs')
+
+        self.assertEqual(
+            "Source package differences between 'Derilucid' and "
+            "parent series",
+            view.label)
+
     def test_batch_includes_needing_attention_only(self):
         # The differences attribute includes differences needing
         # attention only.
-        derived_series, parent_series = self._create_child_and_parent()
+        derived_series, parent_series = self._createChildAndParent()
         current_difference = self.factory.makeDistroSeriesDifference(
             derived_series=derived_series)
         self.factory.makeDistroSeriesDifference(
@@ -671,7 +754,7 @@
 
     def test_batch_includes_different_versions_only(self):
         # The view contains differences of type DIFFERENT_VERSIONS only.
-        derived_series, parent_series = self._create_child_and_parent()
+        derived_series, parent_series = self._createChildAndParent()
         different_versions_diff = self.factory.makeDistroSeriesDifference(
             derived_series=derived_series)
         self.factory.makeDistroSeriesDifference(
@@ -687,7 +770,7 @@
 
     def test_template_includes_help_link(self):
         # The help link for popup help is included.
-        derived_series, parent_series = self._create_child_and_parent()
+        derived_series, parent_series = self._createChildAndParent()
         set_derived_series_ui_feature_flag(self)
         view = create_initialized_view(
             derived_series, '+localpackagediffs')
@@ -699,7 +782,7 @@
 
     def test_diff_row_includes_last_comment_only(self):
         # The most recent comment is rendered for each difference.
-        derived_series, parent_series = self._create_child_and_parent()
+        derived_series, parent_series = self._createChildAndParent()
         difference = self.factory.makeDistroSeriesDifference(
             derived_series=derived_series)
         difference.addComment(difference.owner, "Earlier comment")
@@ -721,7 +804,7 @@
 
     def test_diff_row_links_to_extra_details(self):
         # The source package name links to the difference details.
-        derived_series, parent_series = self._create_child_and_parent()
+        derived_series, parent_series = self._createChildAndParent()
         difference = self.factory.makeDistroSeriesDifference(
             derived_series=derived_series)
 
@@ -737,11 +820,49 @@
         self.assertEqual(1, len(links))
         self.assertEqual(difference.source_package_name.name, links[0].string)
 
+    def test_multiple_parents_display(self):
+        package_name = 'package-1'
+        other_parent_series = self.factory.makeDistroSeries(name='other')
+        derived_series, parent_series = self._createChildAndParents(
+            other_parent_series=other_parent_series)
+        versions = {
+            'base': u'1.0',
+            'derived': u'1.0derived1',
+            'parent': u'1.0-1',
+        }
+
+        self.factory.makeDistroSeriesDifference(
+            versions=versions,
+            parent_series=other_parent_series,
+            source_package_name_str=package_name,
+            derived_series=derived_series)
+        self.factory.makeDistroSeriesDifference(
+            versions=versions,
+            parent_series=parent_series,
+            source_package_name_str=package_name,
+            derived_series=derived_series)
+        set_derived_series_ui_feature_flag(self)
+        view = create_initialized_view(
+            derived_series, '+localpackagediffs')
+        multiple_parents_matches = soupmatchers.HTMLContains(
+            soupmatchers.Tag(
+                "Parent table header", 'th',
+                text=re.compile("\s*Parent\s")),
+            soupmatchers.Tag(
+                "Parent version table header", 'th',
+                text=re.compile("\s*Parent version\s*")),
+            soupmatchers.Tag(
+                "Parent name", 'a',
+                attrs={'class': 'parent-name'},
+                text=re.compile("\s*Other\s*")),
+             )
+        self.assertThat(view.render(), multiple_parents_matches)
+
     def test_diff_row_shows_version_attached(self):
         # The +localpackagediffs page shows the version attached to the
         # DSD and not the last published version (bug=745776).
         package_name = 'package-1'
-        derived_series, parent_series = self._create_child_and_parent()
+        derived_series, parent_series = self._createChildAndParent()
         versions = {
             'base': u'1.0',
             'derived': u'1.0derived1',
@@ -786,7 +907,7 @@
         # The +localpackagediffs page shows only the version (no link)
         # if we fail to fetch the published version.
         package_name = 'package-1'
-        derived_series, parent_series = self._create_child_and_parent()
+        derived_series, parent_series = self._createChildAndParent()
         versions = {
             'base': u'1.0',
             'derived': u'1.0derived1',
@@ -824,11 +945,12 @@
         self.assertEqual(versions['parent'], parent_span[0].string.strip())
 
 
-class TestDistroSeriesLocalDifferencesFunctional(TestCaseWithFactory):
+class TestDistroSeriesLocalDifferencesFunctional(DistroSeriesDifferenceMixin,
+                                                 TestCaseWithFactory):
 
     layer = LaunchpadFunctionalLayer
 
-    def _create_child_and_parent(self):
+    def _createChildAndParent(self):
         parent_series = self.factory.makeDistroSeries(name='lucid')
         derived_series = self.factory.makeDistroSeries(name='derilucid')
         self.factory.makeDistroSeriesParent(
@@ -836,8 +958,10 @@
         return (derived_series, parent_series)
 
     def test_higher_radio_mentions_parent(self):
+        # The user is shown an option to display only the blacklisted
+        # package with a higer version than in the parent.
         set_derived_series_ui_feature_flag(self)
-        derived_series, parent_series = self._create_child_and_parent()
+        derived_series, parent_series = self._createChildAndParent()
         self.factory.makeDistroSeriesDifference(
             derived_series=derived_series,
             source_package_name_str="my-src-package")
@@ -854,6 +978,25 @@
             )
         self.assertThat(view.render(), radio_option_matches)
 
+    def test_higher_radio_mentions_parents(self):
+        set_derived_series_ui_feature_flag(self)
+        derived_series, parent_series = self._createChildAndParents()
+        self.factory.makeDistroSeriesDifference(
+            derived_series=derived_series,
+            source_package_name_str="my-src-package")
+        view = create_initialized_view(
+            derived_series,
+            '+localpackagediffs')
+
+        radio_title = \
+            "&nbsp;Blacklisted packages with a higher version than in parent"
+        radio_option_matches = soupmatchers.HTMLContains(
+            soupmatchers.Tag(
+                "radio displays parent's name", 'label',
+                text=radio_title),
+            )
+        self.assertThat(view.render(), radio_option_matches)
+
     def _set_source_selection(self, series):
         # Set up source package format selection so that copying will
         # work with the default dsc_format used in
@@ -864,7 +1007,7 @@
     def test_batch_filtered(self):
         # The name_filter parameter allows filtering of packages by name.
         set_derived_series_ui_feature_flag(self)
-        derived_series, parent_series = self._create_child_and_parent()
+        derived_series, parent_series = self._createChildAndParent()
         diff1 = self.factory.makeDistroSeriesDifference(
             derived_series=derived_series,
             source_package_name_str="my-src-package")
@@ -888,7 +1031,7 @@
     def test_batch_non_blacklisted(self):
         # The default filter is all non blacklisted differences.
         set_derived_series_ui_feature_flag(self)
-        derived_series, parent_series = self._create_child_and_parent()
+        derived_series, parent_series = self._createChildAndParent()
         diff1 = self.factory.makeDistroSeriesDifference(
             derived_series=derived_series,
             source_package_name_str="my-src-package")
@@ -916,7 +1059,7 @@
         # field.package_type parameter allows to list only
         # blacklisted differences.
         set_derived_series_ui_feature_flag(self)
-        derived_series, parent_series = self._create_child_and_parent()
+        derived_series, parent_series = self._createChildAndParent()
         blacklisted_diff = self.factory.makeDistroSeriesDifference(
             derived_series=derived_series,
             status=DistroSeriesDifferenceStatus.BLACKLISTED_CURRENT)
@@ -938,7 +1081,7 @@
         # field.package_type parameter allows to list only
         # blacklisted differences with a child's version higher than parent's.
         set_derived_series_ui_feature_flag(self)
-        derived_series, parent_series = self._create_child_and_parent()
+        derived_series, parent_series = self._createChildAndParent()
         blacklisted_diff_higher = self.factory.makeDistroSeriesDifference(
             derived_series=derived_series,
             status=DistroSeriesDifferenceStatus.BLACKLISTED_CURRENT,
@@ -966,7 +1109,7 @@
         # Test that we can search for differences that we marked
         # resolved.
         set_derived_series_ui_feature_flag(self)
-        derived_series, parent_series = self._create_child_and_parent()
+        derived_series, parent_series = self._createChildAndParent()
 
         self.factory.makeDistroSeriesDifference(
             derived_series=derived_series,
@@ -1338,12 +1481,13 @@
             'Parent packagesets')
 
 
-class DistroSerieUniquePackageDiffsTestCase(TestCaseWithFactory):
+class DistroSerieUniquePackageDiffsTestCase(DistroSeriesDifferenceMixin,
+                                            TestCaseWithFactory):
     """Test the distroseries +uniquepackages view."""
 
     layer = LaunchpadZopelessLayer
 
-    def _create_child_and_parent(self):
+    def _createChildAndParent(self):
         derived_series = self.factory.makeDistroSeries(name='derilucid')
         parent_series = self.factory.makeDistroSeries(name='lucid')
         self.factory.makeDistroSeriesParent(
@@ -1353,7 +1497,7 @@
     def test_uniquepackages_differences(self):
         # The view fetches the differences with type
         # UNIQUE_TO_DERIVED_SERIES.
-        derived_series, parent_series = self._create_child_and_parent()
+        derived_series, parent_series = self._createChildAndParent()
 
         missing_type = DistroSeriesDifferenceType.UNIQUE_TO_DERIVED_SERIES
         # Missing blacklisted diff.
@@ -1376,7 +1520,7 @@
     def test_uniquepackages_differences_empty(self):
         # The view is empty if there is no differences with type
         # UNIQUE_TO_DERIVED_SERIES.
-        derived_series, parent_series = self._create_child_and_parent()
+        derived_series, parent_series = self._createChildAndParent()
 
         not_missing_type = DistroSeriesDifferenceType.DIFFERENT_VERSIONS
 

=== modified file 'lib/lp/registry/interfaces/distroseries.py'
--- lib/lp/registry/interfaces/distroseries.py	2011-05-06 11:18:35 +0000
+++ lib/lp/registry/interfaces/distroseries.py	2011-05-09 15:04:00 +0000
@@ -897,6 +897,11 @@
     def getDerivedSeries():
         """Get all `DistroSeries` derived from this one."""
 
+    @operation_returns_collection_of(Interface)
+    @export_read_operation()
+    def getParentSeries():
+        """Get all parent `DistroSeries`."""
+
     @operation_parameters(
         parent_series=Reference(
             schema=Interface, # IDistroSeries

=== modified file 'lib/lp/registry/model/distroseries.py'
--- lib/lp/registry/model/distroseries.py	2011-05-06 11:18:35 +0000
+++ lib/lp/registry/model/distroseries.py	2011-05-09 15:04:00 +0000
@@ -754,7 +754,6 @@
             clauseTables=['SourcePackageRelease',
                           'SourcePackagePublishingHistory']).count()
 
-
         # next update the binary count
         clauseTables = ['DistroArchSeries', 'BinaryPackagePublishingHistory',
                         'BinaryPackageRelease']
@@ -791,12 +790,7 @@
     @property
     def is_derived_series(self):
         """See `IDistroSeries`."""
-        # Circular imports.
-        from lp.registry.interfaces.distroseriesparent import (
-            IDistroSeriesParentSet,
-            )
-        dsps = getUtility(IDistroSeriesParentSet).getByDerivedSeries(self)
-        return not dsps.is_empty()
+        return not self.getParentSeries() == []
 
     @property
     def is_initialising(self):
@@ -2006,6 +2000,15 @@
         getUtility(IInitialiseDistroSeriesJobSource).create(
             self, child, architectures, packagesets, rebuild)
 
+    def getParentSeries(self):
+        """See `IDistroSeriesPublic`."""
+        # Circular imports.
+        from lp.registry.interfaces.distroseriesparent import (
+            IDistroSeriesParentSet,
+            )
+        dsps = getUtility(IDistroSeriesParentSet).getByDerivedSeries(self)
+        return [dsp.parent_series for dsp in dsps]
+
     def getDerivedSeries(self):
         """See `IDistroSeriesPublic`."""
         # Circular imports.

=== modified file 'lib/lp/registry/templates/distroseries-index.pt'
--- lib/lp/registry/templates/distroseries-index.pt	2011-04-15 15:08:20 +0000
+++ lib/lp/registry/templates/distroseries-index.pt	2011-05-09 15:04:00 +0000
@@ -65,10 +65,13 @@
           <div
             tal:replace="structure context/@@+portlet-details"/>
         </div>
-        <div class="yui-u"
-             tal:condition="request/features/soyuz.derived-series-ui.enabled">
-          <div tal:replace="structure context/@@+portlet-derivation" />
-        </div>
+        <tal:derivation
+          tal:condition="request/features/soyuz.derived-series-ui.enabled">
+          <div class="yui-u"
+               tal:condition="python:context.is_derived_series or context.is_initialising">
+            <div tal:replace="structure context/@@+portlet-derivation" />
+          </div>
+        </tal:derivation>
 
         <div class="yui-u">
           <div tal:replace="structure context/@@+portlet-package-summary" />

=== modified file 'lib/lp/registry/templates/distroseries-localdifferences.pt'
--- lib/lp/registry/templates/distroseries-localdifferences.pt	2011-05-06 10:45:50 +0000
+++ lib/lp/registry/templates/distroseries-localdifferences.pt	2011-05-09 15:04:00 +0000
@@ -23,7 +23,6 @@
     <div class="top-portlet" metal:fill-slot="main"
       tal:define="differences view/cached_differences;
                   series_name context/displayname;
-                  parent_name view/parent_series/displayname;
                   can_perform_sync view/canPerformSync;">
       <p><tal:replace replace="structure view/explanation/escapedtext" /></p>
 
@@ -43,8 +42,17 @@
           <thead>
             <tr>
               <th>Source</th>
+              <th tal:condition="python: not(view.has_unique_parent) and view.show_parent_version">
+                Parent
+              </th>
               <th tal:condition="view/show_parent_version">
-                <tal:replace replace="parent_name" /> version</th>
+                <tal:one_parent condition="view/has_unique_parent">
+                  <tal:replace replace="view/unique_parent/displayname" /> version
+                </tal:one_parent>
+                <tal:multiple_parents condition="not: view/has_unique_parent">
+                  Parent version
+                </tal:multiple_parents>
+              </th>
               <th tal:condition="view/show_derived_version">
                 <tal:replace replace="series_name" /> version</th>
               <th tal:condition="view/show_parent_packagesets">
@@ -61,6 +69,7 @@
             <tal:difference repeat="difference differences/batch">
             <tr tal:define="parent_source_pub difference/parent_source_pub;
                             source_pub difference/source_pub;
+                            diff_id difference/id;
                             src_name difference/source_package_name/name;"
                 tal:attributes="class src_name">
 
@@ -68,13 +77,17 @@
                 <input tal:condition="can_perform_sync"
                   name="field.selected_differences" type="checkbox"
                   tal:attributes="
-                    value src_name;
-                    id string:field.selected_differences.${src_name}"/>
+                    value diff_id;
+                    id string:field.selected_differences.${diff_id}"/>
 
                 <a tal:attributes="href difference/fmt:url" class="toggle-extra"
                    tal:content="src_name">Foo</a>
               </td>
-
+              <td tal:condition="python: not(view.has_unique_parent) and view.show_parent_version">
+                <a tal:attributes="href difference/parent_series/fmt:url"
+                   tal:content="difference/parent_series/displayname"
+                   class="parent-name">Warty</a>
+              </td>
               <td tal:condition="view/show_parent_version">
                 <a tal:condition="difference/parent_source_package_release"
                    tal:attributes="href difference/parent_source_package_release/fmt:url"

=== modified file 'lib/lp/registry/templates/distroseries-portlet-derivation.pt'
--- lib/lp/registry/templates/distroseries-portlet-derivation.pt	2011-05-05 04:52:36 +0000
+++ lib/lp/registry/templates/distroseries-portlet-derivation.pt	2011-05-09 15:04:00 +0000
@@ -6,7 +6,12 @@
   tal:define="overview_menu context/menu:overview">
   <tal:is_derived condition="context/is_derived_series">
     <tal:is_initialised condition="not: context/is_initialising">
-    <h2>Derived from <tal:name replace="view/parent_series/displayname"/></h2>
+    <tal:one_parent condition="view/has_unique_parent">
+      <h2>Derived from <tal:name replace="view/unique_parent/displayname"/></h2>
+    </tal:one_parent>
+    <tal:multiple_parents condition="not: view/has_unique_parent">
+      <h2>Derived from <tal:name replace="view/number_of_parents"/> parents</h2>
+    </tal:multiple_parents>
 
       <tal:diffs define="nb_diffs view/num_differences;
                          nb_diffs_in_parent view/num_differences_in_parent;
@@ -15,7 +20,7 @@
           <li tal:condition="nb_diffs">
             <a tal:attributes="href string:${context/fmt:url}/+localpackagediffs"
                class="sprite info"
-               title="Source package differences between this series and his parent">
+               title="Source package differences between this series and his parent(s)">
               <tal:nb_diffs replace="nb_diffs"/> package<tal:plural
                 content="string:s"
                 condition="python:nb_diffs!=1"/> with differences
@@ -25,10 +30,17 @@
             <a tal:attributes="href string:${context/fmt:url}/+missingpackages"
                class="sprite info"
                title="Source packages only in parent series">
-              <tal:nb_diffs replace="nb_diffs_in_parent"/> package<tal:plural
-                content="string:s"
-                condition="python:nb_diffs_in_parent!=1"/> only in <tal:replace
-                  replace="view/parent_series/displayname">Sid</tal:replace>
+              <tal:one_parent condition="view/has_unique_parent">
+                <tal:nb_diffs replace="nb_diffs_in_parent"/> package<tal:plural
+                  content="string:s"
+                  condition="python:nb_diffs_in_parent!=1"/> only in <tal:replace
+                    replace="view/unique_parent/displayname">Sid</tal:replace>
+              </tal:one_parent>
+              <tal:multiple_parents condition="not: view/has_unique_parent">
+                <tal:nb_diffs replace="nb_diffs_in_parent"/> package<tal:plural
+                  content="string:s"
+                  condition="python:nb_diffs_in_parent!=1"/> only in a parent series
+               </tal:multiple_parents>
             </a>
           </li>
           <li tal:condition="nb_diffs_in_child">

